#!/usr/local/bin/perl -w

# Gecko Browser Statistics v1.1.0
# http://gemal.dk/mozilla/geckostats.html
#
# Usage:
# perl geckostats.pl access.log > 200301.html

use strict;
use HTML::Template;

# thousands seperator (1000 -> 1.000). Use "." for Europe and "," for US
my $sep = ".";
# how many entries to show. default is 20
my $showentries = 20;

sub num {
	my ($strg)= @_;
	my ($thousands_sep, $grouping)= ($sep, 3);
	if ($thousands_sep) {
		1 while $strg =~ s/(\d)(\d{$grouping}($|\Q$thousands_sep\E))/$1$thousands_sep$2/o;
	}
	return $strg;
}

sub main {
	my $hittotal = 0;
	my $geckototal = 0;
	my $geckorv;
	my %geckorv;
	my @geckorv;
	my $geckovendor;
	my %geckovendor;
	my @geckovendor;
	my %geckovendorrv;
	my @geckovendorrv;
	my $vendorversion;
	my $vendorrv;
	my %vendoronly;
	my @vendoronly;

	my %platform;
	my @platform;
	my %security;
	my @security;
	my %os;
	my @os;
	my %locale;
	my @locale;

	# grap the month and year from the argument if possible
	my $month;
	my $monthname;
	my $year;
	if ($ARGV[0] && $ARGV[0] =~ /(\d\d\d\d)\/(\d\d)\//) {
		$year =  $1;
		$month = $2;
		my @months = ('January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');
		$monthname = $months[$month-1];
	}

	while (<ARGV>) {
		my $tmp = $_;
		++$hittotal;

		my ($rv, $gecko) = $_ =~ /rv:([^); ]+).*(Gecko)\/[0-9]{8}/;

		if ($gecko) {
			my ($vendor, $version) = $_ =~ /Gecko\/[0-9]{8} ([^\/"]+)\/([^" ]+)"/;
			my ($p, $s, $o, $l) = $_ =~ /Mozilla\/.*? \((?:Compatible; )?(.*?);\s?(.*?);\s?(.*?);\s?(.*?);/;
			$platform{$p} += 1 if $p;
			if ($s && $s =~ /N|U|I/) {
				$security{$s} += 1;
			}
			$os{$o} += 1 if $o;
			$locale{$l} += 1 if $l;

			if (!$vendor) {
				$vendor	= "Mozilla";
				$version = "$rv";
			}
			$vendoronly{$vendor} += 1;

			$vendorversion = "$vendor/$version";
			$vendorrv = "$vendor/$rv";

			if (!$geckorv{$rv}) {
				$geckorv{$rv} = 0;
			}

			if (!$geckovendor{$vendorversion}) {
				$geckovendor{$vendorversion} = 0;
			}

			if (!$geckovendorrv{$vendorrv}) {
				$geckovendorrv{$vendorrv} = 0;
			}

			$geckototal += 1;
			$geckorv{$rv} += 1;
			$geckovendor{$vendorversion} += 1;
			$geckovendorrv{$vendorrv} += 1;

		}
	}

	my $template = HTML::Template->new(filename => 'geckostats.tmpl', die_on_bad_params => 0);
	$template->param(year => $year) if $year;
	$template->param(month => $month) if $month;
	$template->param(monthname => $monthname) if $monthname;
	$template->param(total_hits => num($hittotal));
	$template->param(total_gecko => num($geckototal));
	$template->param(total_gecko_percent => int($geckototal * 100 / $hittotal));

	my $i = 0;
	foreach my $x (sort {$geckorv{$b} <=> $geckorv{$a}} keys %geckorv) {
		last if $i++ > $showentries;
		my %row_data;
		$row_data{rv} = $x;
		$row_data{hits} = num($geckorv{$x});
		$row_data{percent} = int($geckorv{$x} * 100 / $geckototal);
		push(@geckorv, \%row_data);
	}
	$template->param(geckoorv => \@geckorv);

	$i = 0;
	foreach my $x (sort {$geckovendorrv{$b} <=> $geckovendorrv{$a}} keys %geckovendorrv) {
		last if $i++ > $showentries;
		my %row_data;
		$row_data{rv} = $x;
		$row_data{hits} = num($geckovendorrv{$x});
		$row_data{percent} = int($geckovendorrv{$x} * 100 / $geckototal);
		push(@geckovendorrv, \%row_data);
	}
	$template->param(geckovendorrv => \@geckovendorrv);

	$i = 0;
	foreach my $x (sort {$geckovendor{$b} <=> $geckovendor{$a}} (keys %geckovendor)) {
		last if $i++ > $showentries;
		my %row_data;
		$row_data{rv} = $x;
		$row_data{hits} = num($geckovendor{$x});
		$row_data{percent} = int($geckovendor{$x} * 100 / $geckototal);
		push(@geckovendor, \%row_data);
	}
	$template->param(geckovendor => \@geckovendor);

	$i = 0;
	foreach my $x (sort {$vendoronly{$b} <=> $vendoronly{$a}} (keys %vendoronly)) {
		last if $i++ > $showentries;
		my %row_data;
		$row_data{rv} = $x;
		$row_data{hits} = num($vendoronly{$x});
		$row_data{percent} = int($vendoronly{$x} * 100 / $geckototal);
		push(@vendoronly, \%row_data);
	}
	$template->param(vendoronly => \@vendoronly);

	$i = 0;
	foreach my $x (sort {$platform{$b} <=> $platform{$a}} (keys %platform)) {
		last if $i++ > $showentries;
		my %row_data;
		$row_data{rv} = $x;
		$row_data{hits} = num($platform{$x});
		$row_data{percent} = int($platform{$x} * 100 / $geckototal);
		push(@platform, \%row_data);
	}
	$template->param(platform => \@platform);

	$i = 0;
	foreach my $x (sort {$security{$b} <=> $security{$a}} (keys %security)) {
		last if $i++ > $showentries;
		my %row_data;
		$row_data{rv} = $x;
		$row_data{hits} = num($security{$x});
		$row_data{percent} = int($security{$x} * 100 / $geckototal);
		push(@security, \%row_data);
	}
	$template->param(security => \@security);

	$i = 0;
	foreach my $x (sort {$os{$b} <=> $os{$a}} (keys %os)) {
		last if $i++ > $showentries;
		my %row_data;
		$row_data{rv} = $x;
		$row_data{hits} = num($os{$x});
		$row_data{percent} = int($os{$x} * 100 / $geckototal);
		push(@os, \%row_data);
	}
	$template->param(os => \@os);

	$i = 0;
	foreach my $x (sort {$locale{$b} <=> $locale{$a}} (keys %locale)) {
		last if $i++ > $showentries;
		my %row_data;
		$row_data{rv} = $x;
		$row_data{hits} = num($locale{$x});
		$row_data{percent} = int($locale{$x} * 100 / $geckototal);
		push(@locale, \%row_data);
	}
	$template->param(locale => \@locale);

	print $template->output;
}

main();
